<?php

if ( !defined( 'ABSPATH' ) ) {
	exit;
}

if ( !class_exists( 'WCOT_Order_Tags_Bulk_Actions' ) ) {

	class WCOT_Order_Tags_Bulk_Actions {

		public function __construct() {

			add_filter( 'bulk_actions-edit-shop_order', array( $this, 'add' ), 20, 1 ); // 20 so after core bulk actions
			add_filter( 'handle_bulk_actions-edit-shop_order', array( $this, 'handle' ), 10, 3 );
			add_action( 'admin_notices', array( $this, 'notice' ) );

		}

		public function add( $actions ) {

			$order_tags = wcot_order_tags_get_all_order_tags();

			if ( !empty( $order_tags ) ) {

				foreach ( $order_tags as $order_tag ) {

					// translators: %s: order tag name
					$actions['wcot_order_tags_set_' . $order_tag->term_id] = sprintf( __( 'Set order tag %s', 'wcot-order-tags' ), strtolower( $order_tag->name ) );

				}

			}

			return $actions;

		}

		public function handle( $redirect_to, $action, $order_ids ) {

			if ( substr( $action, 0, 20 ) === 'wcot_order_tags_set_' ) {

				$order_tags_set = 0;

				foreach ( $order_ids as $order_id ) {

					$term_id = str_replace( 'wcot_order_tags_set_', '', $action );
					$set_order_tag = wcot_order_tags_set_order_tag( $order_id, $term_id );

					if ( is_array( $set_order_tag ) ) {

						$order_tags_set++;

					}

				}

				$redirect_to = add_query_arg(
					array(
						'wcot_order_tags_set' => $order_tags_set,
					),
					$redirect_to
				);

			}

			return $redirect_to;

		}

		public function notice() {

			global $post_type, $pagenow;

			if ( 'edit.php' !== $pagenow || 'shop_order' !== $post_type ) {

				return;

			}

			if ( isset( $_GET['wcot_order_tags_set'] ) ) {

				$order_tags_set = (int) sanitize_text_field( $_GET['wcot_order_tags_set'] );

				if ( $order_tags_set > 0 ) { ?>

					<div class="notice notice-success">
						<p><?php echo esc_html( $order_tags_set ); ?> <?php esc_html_e( 'order tags set.', 'wcot-order-tags' ); ?></p>
					</div>

				<?php } else { ?>

					<div class="notice notice-error">
						<p><?php esc_html_e( 'No order tags set.', 'wcot-order-tags' ); ?></p>
					</div>
			
					<?php

				}

			}

		}

	}

}
