<?php

if ( !defined( 'ABSPATH' ) ) {
	exit;
}

if ( !class_exists( 'WCOT_Order_Tags_Orders' ) ) {

	class WCOT_Order_Tags_Orders {

		public function __construct() {

			add_filter( 'manage_edit-shop_order_columns', array( $this, 'columns' ) );
			add_action( 'manage_shop_order_posts_custom_column', array( $this, 'column' ) );
			add_action( 'woocommerce_admin_order_data_after_order_details', array( $this, 'after_order_details' ) );

		}

		public function columns( $columns ) {

			$order_total = $columns['order_total'];
			unset( $columns['order_total'] );

			$columns['wcot_order_tags'] = __( 'Tags', 'wcot-order-tags' );
			$columns['order_total'] = $order_total;

			return $columns;

		}

		public function column( $column ) {

			global $post;

			if ( 'wcot_order_tags' == $column ) {

				$order_tags = wcot_order_tags_get_order_tags( $post->ID );

				if ( !empty( $order_tags ) ) {

					foreach ( $order_tags as $order_tag ) {

						echo wp_kses_post( wcot_order_tags_get_order_tag_html( $order_tag ) );

					}

				} else {

					esc_html_e( '—', 'wcot-order-tags' );

				}

			}

		}

		public function after_order_details( $order ) {

			$order_id = $order->get_id();

			if ( !empty( $order_id ) ) {

				// The shop_order condition below is here because there was a period where order tags were incorrectly being assigned to subscription based orders (the shop_subscription post type orders not the individual renewal orders), this stops any tags applied during this period from displaying on subscription orders, if the user has since/does a bulk tag set the invisible data that still exists would be removed anyway due to the amended conditions since in set_tags(), but this condition is just here to ensure the user does not see these if not

				if ( 'shop_order' == get_post_type( $order_id ) ) {

					$order_tags = wcot_order_tags_get_order_tags( $order_id );

					if ( !empty( $order_tags ) ) {
		
						echo '<div class="wcot-order-tags-after-order-details">';
		
						foreach ( $order_tags as $order_tag ) {
		
							echo wp_kses_post( wcot_order_tags_get_order_tag_html( $order_tag ) );
		
						}
		
						echo '</div>';
		
					}

				}

			}

		}

	}

}
