<?php

if ( !defined( 'ABSPATH' ) ) {
	exit;
}

if ( !class_exists( 'WCOT_Order_Tags_Taxonomy' ) ) {

	class WCOT_Order_Tags_Taxonomy {

		public function __construct() {

			add_action( 'init', array( $this, 'register' ) );
			add_action( 'wcot_order_tag_edit_form_fields', array( $this, 'fields' ) );
			add_action( 'wp_ajax_wcot_order_tags_fields_select2_set_if_customer_value', array( $this, 'fields_select2_set_if_customer_value' ) );
			add_action( 'wp_ajax_wcot_order_tags_fields_select2_set_if_product_value', array( $this, 'fields_select2_set_if_product_value' ) );
			add_action( 'wp_ajax_wcot_order_tags_fields_select2_set_if_product_category_value', array( $this, 'fields_select2_set_if_product_category_value' ) );
			add_action( 'wp_ajax_wcot_order_tags_fields_select2_set_if_product_tag_value', array( $this, 'fields_select2_set_if_product_tag_value' ) );
			add_action( 'edited_wcot_order_tag', array( $this, 'save' ) );
			add_action( 'manage_edit-wcot_order_tag_columns', array( $this, 'columns' ) );

		}

		public function register() {

			$labels = array(
				'name'							=> __( 'Order Tags', 'wcot-order-tags' ),
				'singular_name'					=> __( 'Order Tag', 'wcot-order-tags' ),
				'menu_name'						=> __( 'Order Tags', 'wcot-order-tags' ),
				'all_items'						=> __( 'All', 'wcot-order-tags' ) . ' ' . __( 'Order Tags', 'wcot-order-tags' ),
				'parent_item'					=> __( 'Parent', 'wcot-order-tags' ) . ' ' . __( 'Order Tag', 'wcot-order-tags' ),
				'parent_item_colon'				=> __( 'Parent', 'wcot-order-tags' ) . ' ' . __( 'Order Tag', 'wcot-order-tags' ) . ' ' . __( ':', 'wcot-order-tags' ),
				'new_item_name'					=> __( 'New', 'wcot-order-tags' ) . ' ' . __( 'Order Tag', 'wcot-order-tags' ),
				'add_new_item'					=> __( 'Add New', 'wcot-order-tags' ) . ' ' . __( 'Order Tag', 'wcot-order-tags' ),
				'edit_item'						=> __( 'Edit', 'wcot-order-tags' ) . ' ' . __( 'Order Tag', 'wcot-order-tags' ),
				'update_item'					=> __( 'Update', 'wcot-order-tags' ) . ' ' . __( 'Order Tag', 'wcot-order-tags' ),
				'view_item'						=> __( 'View', 'wcot-order-tags' ) . ' ' . __( 'Order Tags', 'wcot-order-tags' ),
				'separate_items_with_commas'	=> __( 'Separate', 'wcot-order-tags' ) . ' ' . strtolower( __( 'Order Tags', 'wcot-order-tags' ) ) . ' ' . __( 'with commas', 'wcot-order-tags' ),
				'add_or_remove_items'			=> __( 'Add or remove', 'wcot-order-tags' ) . ' ' . strtolower( __( 'Order Tags', 'wcot-order-tags' ) ),
				'choose_from_most_used'			=> __( 'Choose from the most used', 'wcot-order-tags' ),
				'popular_items'					=> __( 'Popular', 'wcot-order-tags' ) . ' ' . __( 'Order Tags', 'wcot-order-tags' ),
				'search_items'					=> __( 'Search', 'wcot-order-tags' ) . ' ' . __( 'Order Tags', 'wcot-order-tags' ),
				'not_found'						=> __( 'Not Found', 'wcot-order-tags' ),
				'no_terms'						=> __( 'No', 'wcot-order-tags' ) . ' ' . strtolower( __( 'Order Tags', 'wcot-order-tags' ) ),
				'items_list'					=> __( 'Order Tags', 'wcot-order-tags' ) . ' ' . __( 'list', 'wcot-order-tags' ),
				'items_list_navigation'			=> __( 'Order Tags', 'wcot-order-tags' ) . ' ' . __( 'list navigation', 'wcot-order-tags' ),
			);

			register_taxonomy(
				'wcot_order_tag',
				'shop_order',
				apply_filters(
					'wcot_order_tag_taxonomy_args',
					array(
						'labels'		=> $labels,
						'public'		=> false,
						'hierarchical'	=> true,
						'show_ui'		=> true,
					)
				)
			);

		}

		public function fields( $term ) {

			// Initial data

			$term_id = $term->term_id;

			$customer_roles = get_editable_roles();
			$order_statuses = wc_get_order_statuses();
			$payment_methods = WC()->payment_gateways->payment_gateways();

			$shipping_zones = WC_Shipping_Zones::get_zones();
			$shipping_methods = WC()->shipping->load_shipping_methods();
			$shipping_method_instances = array_map(
				function( $shipping_zone ) {
					return $shipping_zone['shipping_methods'];
				},
				$shipping_zones
			);

			// WooCommerce Subscriptions active

			require_once ABSPATH . 'wp-admin/includes/plugin.php'; // Ensures is_plugin_active() can be used here

			if ( is_plugin_active( 'woocommerce-subscriptions/woocommerce-subscriptions.php' ) ) { // If WooCommerce Subscriptions is active, works for standalone and multisite network

				$woocommerce_subscriptions_active = true;

			} else {

				$woocommerce_subscriptions_active = false;

			}

			// Color

			$color = get_term_meta( $term_id, 'wcot_order_tags_color', true );

			// Set on order placed

			$set_on_order_placed = get_term_meta( $term_id, 'wcot_order_tags_set_on_order_placed', true );

			// Set on order update

			$set_on_order_update = get_term_meta( $term_id, 'wcot_order_tags_set_on_order_update', true );

			// Set on order subscription renewal

			$set_on_order_subscription_renewal = get_term_meta( $term_id, 'wcot_order_tags_set_on_order_subscription_renewal', true );

			// Set if customer

			$set_if_customer = get_term_meta( $term_id, 'wcot_order_tags_set_if_customer', true );
			$set_if_customer_value = get_term_meta( $term_id, 'wcot_order_tags_set_if_customer_value', true );
			$set_if_customer_value = ( !empty( $set_if_customer_value ) ? $set_if_customer_value : array() );

			// Set if customer order count
			
			$set_if_customer_order_count = get_term_meta( $term_id, 'wcot_order_tags_set_if_customer_order_count', true );
			$set_if_customer_order_count_operator = get_term_meta( $term_id, 'wcot_order_tags_set_if_customer_order_count_operator', true );
			$set_if_customer_order_count_value = get_term_meta( $term_id, 'wcot_order_tags_set_if_customer_order_count_value', true );

			// Set if customer role

			$set_if_customer_role = get_term_meta( $term_id, 'wcot_order_tags_set_if_customer_role', true );
			$set_if_customer_role_value = get_term_meta( $term_id, 'wcot_order_tags_set_if_customer_role_value', true );
			$set_if_customer_role_value = ( !empty( $set_if_customer_role_value ) ? $set_if_customer_role_value : array() );

			// Set if item count

			$set_if_item_count = get_term_meta( $term_id, 'wcot_order_tags_set_if_item_count', true );
			$set_if_item_count_operator = get_term_meta( $term_id, 'wcot_order_tags_set_if_item_count_operator', true );
			$set_if_item_count_value = get_term_meta( $term_id, 'wcot_order_tags_set_if_item_count_value', true );

			// Set if meta data

			$set_if_meta_data = get_term_meta( $term_id, 'wcot_order_tags_set_if_meta_data', true );
			$set_if_meta_data_value = get_term_meta( $term_id, 'wcot_order_tags_set_if_meta_data_value', true );

			// Set if order status

			$set_if_order_status = get_term_meta( $term_id, 'wcot_order_tags_set_if_order_status', true );
			$set_if_order_status_value = get_term_meta( $term_id, 'wcot_order_tags_set_if_order_status_value', true );
			$set_if_order_status_value = ( !empty( $set_if_order_status_value ) ? $set_if_order_status_value : array() );

			// Set if order total

			$set_if_order_total = get_term_meta( $term_id, 'wcot_order_tags_set_if_order_total', true );
			$set_if_order_total_operator = get_term_meta( $term_id, 'wcot_order_tags_set_if_order_total_operator', true );
			$set_if_order_total_value = get_term_meta( $term_id, 'wcot_order_tags_set_if_order_total_value', true );

			// Set if payment method

			$set_if_payment_method = get_term_meta( $term_id, 'wcot_order_tags_set_if_payment_method', true );
			$set_if_payment_method_value = get_term_meta( $term_id, 'wcot_order_tags_set_if_payment_method_value', true );
			$set_if_payment_method_value = ( !empty( $set_if_payment_method_value ) ? $set_if_payment_method_value : array() );

			// Set if product

			$set_if_product = get_term_meta( $term_id, 'wcot_order_tags_set_if_product', true );
			$set_if_product_value = get_term_meta( $term_id, 'wcot_order_tags_set_if_product_value', true );
			$set_if_product_value = ( !empty( $set_if_product_value ) ? $set_if_product_value : array() );

			// Set if product category

			$set_if_product_category = get_term_meta( $term_id, 'wcot_order_tags_set_if_product_category', true );
			$set_if_product_category_value = get_term_meta( $term_id, 'wcot_order_tags_set_if_product_category_value', true );
			$set_if_product_category_value = ( !empty( $set_if_product_category_value ) ? $set_if_product_category_value : array() );

			// Set if product meta data

			$set_if_product_meta_data = get_term_meta( $term_id, 'wcot_order_tags_set_if_product_meta_data', true );
			$set_if_product_meta_data_value = get_term_meta( $term_id, 'wcot_order_tags_set_if_product_meta_data_value', true );

			// Set if product tag

			$set_if_product_tag = get_term_meta( $term_id, 'wcot_order_tags_set_if_product_tag', true );
			$set_if_product_tag_value = get_term_meta( $term_id, 'wcot_order_tags_set_if_product_tag_value', true );
			$set_if_product_tag_value = ( !empty( $set_if_product_tag_value ) ? $set_if_product_tag_value : array() );

			// Set if shipping method

			$set_if_shipping_method = get_term_meta( $term_id, 'wcot_order_tags_set_if_shipping_method', true );
			$set_if_shipping_method_value = get_term_meta( $term_id, 'wcot_order_tags_set_if_shipping_method_value', true );
			$set_if_shipping_method_value = ( !empty( $set_if_shipping_method_value ) ? $set_if_shipping_method_value : array() );

			// Set if shipping method instance

			$set_if_shipping_method_instance = get_term_meta( $term_id, 'wcot_order_tags_set_if_shipping_method_instance', true );
			$set_if_shipping_method_instance_value = get_term_meta( $term_id, 'wcot_order_tags_set_if_shipping_method_instance_value', true );
			$set_if_shipping_method_instance_value = ( !empty( $set_if_shipping_method_instance_value ) ? $set_if_shipping_method_instance_value : array() );

			// Set if shipping zone

			$set_if_shipping_zone = get_term_meta( $term_id, 'wcot_order_tags_set_if_shipping_zone', true );
			$set_if_shipping_zone_value = get_term_meta( $term_id, 'wcot_order_tags_set_if_shipping_zone_value', true );
			$set_if_shipping_zone_value = ( !empty( $set_if_shipping_zone_value ) ? $set_if_shipping_zone_value : array() );

			// Set if mode

			$set_if_mode = get_term_meta( $term_id, 'wcot_order_tags_set_if_mode', true );

			// Meta data field info

			$meta_data_field_info = sprintf(
				// translators: %s: operators
				esc_html__( 'One meta data condition per line. Format is meta_key|operator|meta_value. Meta key must be singular. Operator can be %s. Meta value can not be empty. All the conditions entered must be met to set. Example usage:', 'wcot-order-tags' ),
				'<code>=</code>, <code>!=</code>, <code>>=</code>, <code>></code>, <code><=</code>, <code><</code> ' . esc_html__( 'or', 'wcot-order-tags' ) . ' <code>contains</code>'
			);

			// Nonce

			wp_nonce_field( 'wcot_order_tags_save', 'wcot_order_tags_save_nonce' );

			?>

			<table class="wcot-order-tags-fields form-table">
				<tbody>
					<tr class="form-field">
						<th scope="row">
							<label for="wcot-order-tags-color"><?php esc_html_e( 'Color', 'wcot-order-tags' ); ?></label>
						</th>
						<td>
							<input type="text" id="wcot-order-tags-color" class="wcot-order-tags-color-picker" name="wcot_order_tags_color" value="<?php echo esc_html( $color ); ?>">
							<p class="description">
								<?php esc_html_e( 'Sets the color of the tag. White text is used so a dark color is recommended.', 'wcot-order-tags' ); ?><br>
								<?php
								// translators: %s: color default
								echo sprintf( esc_html__( 'If empty defaults to %s.', 'wcot-order-tags' ), '<code>' . esc_html( apply_filters( 'wcot_order_tags_color_default', '#000000' ) ) . '</code>' );
								?>
							</p>
						</td>
					</tr>
					<tr class="form-field">
						<th scope="row">
							<label for="wcot-order-tags-set-on"><?php esc_html_e( 'Set On', 'wcot-order-tags' ); ?></label>
							<p class="description"><?php esc_html_e( '"Set If" must also be set', 'wcot-order-tags' ); ?></p>
						</th>
						<td>
							<div class="wcot-order-tags-checkbox-wrap">
								<label><input type="checkbox" name="wcot_order_tags_set_on_order_placed"<?php echo ( 'on' == $set_on_order_placed ? ' checked' : '' ); ?>><?php esc_html_e( 'Order placed', 'wcot-order-tags' ); ?></label>
								<p class="description">
									<?php esc_html_e( 'Sets the tag when an order is placed via checkout or adding an order via dashboard. Some payment methods may also immediately update the order after it has been placed e.g. to change order status once payment confirmed.', 'wcot-order-tags' ); ?>
								</p>
							</div>
							<div class="wcot-order-tags-checkbox-wrap">
								<label><input type="checkbox" name="wcot_order_tags_set_on_order_update"<?php echo ( 'on' == $set_on_order_update ? ' checked' : '' ); ?>><?php esc_html_e( 'Order update', 'wcot-order-tags' ); ?></label>
								<p class="description"><?php esc_html_e( 'Sets the tag when an order is updated. If a tag is attempted to be removed manually from an order this may cause it to be restored automatically upon update if the "Set If" conditions are met.', 'wcot-order-tags' ); ?></p>
							</div>
							<?php if ( true == $woocommerce_subscriptions_active ) { ?>
								<div class="wcot-order-tags-checkbox-wrap">
									<label><input type="checkbox" name="wcot_order_tags_set_on_order_subscription_renewal"<?php echo ( 'on' == $set_on_order_subscription_renewal ? ' checked' : '' ); ?>><?php esc_html_e( 'Order subscription renewal', 'wcot-order-tags' ); ?></label>
									<p class="description"><?php esc_html_e( 'Sets the tag when a subscription renewal order gets created.', 'wcot-order-tags' ); ?></p>
								</div>
							<?php } ?>
						</td>
					</tr>
					<tr class="form-field">
						<th scope="row">
							<?php esc_html_e( 'Set If', 'wcot-order-tags' ); ?>
							<p class="description"><?php esc_html_e( '"Set On" must also be set', 'wcot-order-tags' ); ?></p>
						</th>
						<td>
							<div class="wcot-order-tags-checkbox-wrap">
								<label>
									<input type="checkbox" class="wcot-order-tags-checkbox-expanding" name="wcot_order_tags_set_if_customer"<?php echo ( 'on' == $set_if_customer ? ' checked' : '' ); ?>><?php esc_html_e( 'Customer', 'wcot-order-tags' ); ?>
								</label>
								<?php // translators: %s: filter name ?>
								<p class="description"><?php echo sprintf( esc_html__( 'Sets the tag if the order is by a specific customer(s). By default customers are shown for selection here if they have the "customer" user role, use the %s filter to include customers with custom roles.', 'wcot-order-tags' ), '<code>wcot_order_tags_customer_roles</code>' ); ?></p>
								<div class="wcot-order-tags-checkbox-expanded">
									<label>
										<?php esc_html_e( 'Customer(s)', 'wcot-order-tags' ); ?><br>
										<select class="wcot-order-tags-select2-ajax-set-if-customer-value" name="wcot_order_tags_set_if_customer_value[]" multiple>
											<?php
											if ( !empty( $set_if_customer_value ) ) {
												foreach ( $set_if_customer_value as $set_if_customer_value_id ) {
													$set_if_customer_value_customer = get_user_by( 'id', $set_if_customer_value_id );
													if ( !empty( $set_if_customer_value_customer ) ) {
														?>
														<option value="<?php echo esc_html( $set_if_customer_value_id ); ?>" selected><?php echo esc_html( $set_if_customer_value_customer->display_name ) . ' ' . esc_html__( '-', 'wcot-order-tags' ) . ' ' . esc_html( $set_if_customer_value_customer->user_email ) . ' ' . esc_html__( '-', 'wcot-order-tags' ) . ' ' . esc_html__( 'ID:', 'wcot-order-tags' ) . ' ' . esc_html( $set_if_customer_value_id ); ?></option>
														<?php
													}
												}
											}
											?>
										</select>
									</label>
								</div>
							</div>
							<div class="wcot-order-tags-checkbox-wrap">
								<label>
									<input type="checkbox" class="wcot-order-tags-checkbox-expanding" name="wcot_order_tags_set_if_customer_order_count"<?php echo ( 'on' == $set_if_customer_order_count ? ' checked' : '' ); ?>><?php esc_html_e( 'Customer order count', 'wcot-order-tags' ); ?>
								</label>
								<p class="description"><?php esc_html_e( 'Sets the tag based on the amount of orders the customer has placed including the current order. For accuracy it is recommended this is only set on order placed as the customer order count may have increased by the time the order is updated.', 'wcot-order-tags' ); ?></p>
								<div class="wcot-order-tags-checkbox-expanded">
									<label class="wcot-order-tags-label-float">
										<?php esc_html_e( 'Operator', 'wcot-order-tags' ); ?><br>
										<select name="wcot_order_tags_set_if_customer_order_count_operator">
											<option<?php echo ( '=' == $set_if_customer_order_count_operator ? ' selected' : '' ); ?>>
												=
											</option>
											<option<?php echo ( '!=' == $set_if_customer_order_count_operator ? ' selected' : '' ); ?>>
												!=
											</option>
											<option<?php echo ( '>=' == $set_if_customer_order_count_operator ? ' selected' : '' ); ?>>
												>=
											</option>
											<option<?php echo ( '>' == $set_if_customer_order_count_operator ? ' selected' : '' ); ?>>
												>
											</option>
											<option<?php echo ( '<=' == $set_if_customer_order_count_operator ? ' selected' : '' ); ?>>
												<=
											</option>
											<option<?php echo ( '<' == $set_if_customer_order_count_operator ? ' selected' : '' ); ?>>
												<
											</option>
										</select>
									</label>
									<label class="wcot-order-tags-label-float">
										<?php esc_html_e( 'Customer order count', 'wcot-order-tags' ); ?><br>
										<input type="number" name="wcot_order_tags_set_if_customer_order_count_value" value="<?php echo esc_html( $set_if_customer_order_count_value ); ?>" min="0">
									</label>
								</div>
							</div>
							<div class="wcot-order-tags-checkbox-wrap">
								<label>
									<input type="checkbox" class="wcot-order-tags-checkbox-expanding" name="wcot_order_tags_set_if_customer_role"<?php echo ( 'on' == $set_if_customer_role ? ' checked' : '' ); ?>><?php esc_html_e( 'Customer role', 'wcot-order-tags' ); ?>
								</label>
								<p class="description"><?php esc_html_e( 'Sets the tag if the order is by a customer with a specific role(s).', 'wcot-order-tags' ); ?></p>
								<div class="wcot-order-tags-checkbox-expanded">
									<label>
										<?php esc_html_e( 'Customer role(s)', 'wcot-order-tags' ); ?><br>
										<select class="wcot-order-tags-select2" name="wcot_order_tags_set_if_customer_role_value[]" multiple>
											<?php foreach ( $customer_roles as $customer_role_id => $customer_role ) { ?>
												<option value="<?php echo esc_html( $customer_role_id ); ?>"<?php echo ( in_array( $customer_role_id, $set_if_customer_role_value ) ? ' selected' : '' ); ?>><?php echo esc_html( $customer_role['name'] ) . ' ' . esc_html__( '-', 'wcot-order-tags' ) . ' ' . esc_html__( 'ID:', 'wcot-order-tags' ) . ' ' . esc_html( $customer_role_id ); ?></option>
											<?php } ?>
										</select>
									</label>
								</div>
							</div>
							<div class="wcot-order-tags-checkbox-wrap">
								<label>
									<input type="checkbox" class="wcot-order-tags-checkbox-expanding" name="wcot_order_tags_set_if_item_count"<?php echo ( 'on' == $set_if_item_count ? ' checked' : '' ); ?>><?php esc_html_e( 'Item count', 'wcot-order-tags' ); ?>
								</label>
								<p class="description"><?php esc_html_e( 'Sets the tag based on the amount of items within the order, including multiple quantities of the same item.', 'wcot-order-tags' ); ?></p>
								<div class="wcot-order-tags-checkbox-expanded">
									<label class="wcot-order-tags-label-float">
										<?php esc_html_e( 'Operator', 'wcot-order-tags' ); ?><br>
										<select name="wcot_order_tags_set_if_item_count_operator">
											<option<?php echo ( '=' == $set_if_item_count_operator ? ' selected' : '' ); ?>>
												=
											</option>
											<option<?php echo ( '!=' == $set_if_item_count_operator ? ' selected' : '' ); ?>>
												!=
											</option>
											<option<?php echo ( '>=' == $set_if_item_count_operator ? ' selected' : '' ); ?>>
												>=
											</option>
											<option<?php echo ( '>' == $set_if_item_count_operator ? ' selected' : '' ); ?>>
												>
											</option>
											<option<?php echo ( '<=' == $set_if_item_count_operator ? ' selected' : '' ); ?>>
												<=
											</option>
											<option<?php echo ( '<' == $set_if_item_count_operator ? ' selected' : '' ); ?>>
												<
											</option>
										</select>
									</label>
									<label class="wcot-order-tags-label-float">
										<?php esc_html_e( 'Item count', 'wcot-order-tags' ); ?><br>
										<input type="number" name="wcot_order_tags_set_if_item_count_value" value="<?php echo esc_html( $set_if_item_count_value ); ?>" min="0">
									</label>
								</div>
							</div>
							<div class="wcot-order-tags-checkbox-wrap">
								<label>
									<input type="checkbox" class="wcot-order-tags-checkbox-expanding" name="wcot_order_tags_set_if_meta_data"<?php echo ( 'on' == $set_if_meta_data ? ' checked' : '' ); ?>><?php esc_html_e( 'Meta data', 'wcot-order-tags' ); ?>
								</label>
								<p class="description"><?php esc_html_e( 'Sets the tag if the order has specific meta data(s).', 'wcot-order-tags' ); ?></p>
								<div class="wcot-order-tags-checkbox-expanded">
									<label>
										<?php esc_html_e( 'Meta data(s)', 'wcot-order-tags' ); ?><br>
										<textarea name="wcot_order_tags_set_if_meta_data_value"><?php echo wp_kses_post( $set_if_meta_data_value ); ?></textarea>
									</label>
									<p class="wcot-order-tags-info"><?php echo wp_kses_post( $meta_data_field_info ); ?></p>
									<code>_shipping_postcode|contains|AB1
									_warranty_fee|>|20.00
									_warranty_sold|=|yes
									</code><?php // This is intentionaly indented this way to display correctly ?>
								</div>
							</div>
							<div class="wcot-order-tags-checkbox-wrap">
								<label>
									<input type="checkbox" class="wcot-order-tags-checkbox-expanding" name="wcot_order_tags_set_if_order_status"<?php echo ( 'on' == $set_if_order_status ? ' checked' : '' ); ?>><?php esc_html_e( 'Order status', 'wcot-order-tags' ); ?>
								</label>
								<p class="description"><?php esc_html_e( 'Sets the tag if the order has a specific order status(es). It is recommended this is used in conjunction with other "Set If" conditions than individually.', 'wcot-order-tags' ); ?></p>
								<div class="wcot-order-tags-checkbox-expanded">
									<label>
										<?php esc_html_e( 'Order status(es)', 'wcot-order-tags' ); ?><br>
										<select class="wcot-order-tags-select2" name="wcot_order_tags_set_if_order_status_value[]" multiple>
											<?php foreach ( $order_statuses as $order_status_id => $order_status_name ) { ?>
												<option value="<?php echo esc_html( $order_status_id ); ?>"<?php echo ( in_array( $order_status_id, $set_if_order_status_value ) ? ' selected' : '' ); ?>><?php echo esc_html( $order_status_name ) . ' ' . esc_html__( '-', 'wcot-order-tags' ) . ' ' . esc_html__( 'ID:', 'wcot-order-tags' ) . ' ' . esc_html( $order_status_id ); ?></option>
											<?php } ?>
										</select>
									</label>
								</div>
							</div>
							<div class="wcot-order-tags-checkbox-wrap">
								<label>
									<input type="checkbox" class="wcot-order-tags-checkbox-expanding" name="wcot_order_tags_set_if_order_total"<?php echo ( 'on' == $set_if_order_total ? ' checked' : '' ); ?>><?php esc_html_e( 'Order total', 'wcot-order-tags' ); ?>
								</label>
								<p class="description"><?php esc_html_e( 'Sets the tag based on the order total.', 'wcot-order-tags' ); ?></p>
								<div class="wcot-order-tags-checkbox-expanded">
									<label class="wcot-order-tags-label-float">
										<?php esc_html_e( 'Operator', 'wcot-order-tags' ); ?><br>
										<select name="wcot_order_tags_set_if_order_total_operator">
											<option<?php echo ( '=' == $set_if_order_total_operator ? ' selected' : '' ); ?>>
												=
											</option>
											<option<?php echo ( '!=' == $set_if_order_total_operator ? ' selected' : '' ); ?>>
												!=
											</option>
											<option<?php echo ( '>=' == $set_if_order_total_operator ? ' selected' : '' ); ?>>
												>=
											</option>
											<option<?php echo ( '>' == $set_if_order_total_operator ? ' selected' : '' ); ?>>
												>
											</option>
											<option<?php echo ( '<=' == $set_if_order_total_operator ? ' selected' : '' ); ?>>
												<=
											</option>
											<option<?php echo ( '<' == $set_if_order_total_operator ? ' selected' : '' ); ?>>
												<
											</option>
										</select>
									</label>
									<label class="wcot-order-tags-label-float">
										<?php esc_html_e( 'Order total', 'wcot-order-tags' ); ?><br>
										<input type="number" name="wcot_order_tags_set_if_order_total_value" value="<?php echo esc_html( $set_if_order_total_value ); ?>" min="0">
									</label>
								</div>
							</div>
							<div class="wcot-order-tags-checkbox-wrap">
								<label>
									<input type="checkbox" class="wcot-order-tags-checkbox-expanding" name="wcot_order_tags_set_if_payment_method"<?php echo ( 'on' == $set_if_payment_method ? ' checked' : '' ); ?>><?php esc_html_e( 'Payment method', 'wcot-order-tags' ); ?>
								</label>
								<p class="description"><?php esc_html_e( 'Sets the tag based on the payment method(s) used.', 'wcot-order-tags' ); ?></p>
								<div class="wcot-order-tags-checkbox-expanded">
									<label>
										<?php esc_html_e( 'Payment method(s)', 'wcot-order-tags' ); ?><br>
										<select class="wcot-order-tags-select2" name="wcot_order_tags_set_if_payment_method_value[]" multiple>
											<?php foreach ( $payment_methods as $payment_method_id => $payment_method ) { ?>
												<option value="<?php echo esc_html( $payment_method_id ); ?>"<?php echo ( in_array( $payment_method_id, $set_if_payment_method_value ) ? ' selected' : '' ); ?>><?php echo esc_html( $payment_method->method_title ) . ' ' . esc_html__( '-', 'wcot-order-tags' ) . ' ' . esc_html__( 'ID:', 'wcot-order-tags' ) . ' ' . esc_html( $payment_method_id ); ?></option>
											<?php } ?>
										</select>
									</label>
								</div>
							</div>
							<div class="wcot-order-tags-checkbox-wrap">
								<label>
									<input type="checkbox" class="wcot-order-tags-checkbox-expanding" name="wcot_order_tags_set_if_product"<?php echo ( 'on' == $set_if_product ? ' checked' : '' ); ?>><?php esc_html_e( 'Product', 'wcot-order-tags' ); ?>
								</label>
								<p class="description"><?php esc_html_e( 'Sets the tag if the order contains a specific product(s).', 'wcot-order-tags' ); ?></p>
								<div class="wcot-order-tags-checkbox-expanded">
									<label>
										<?php esc_html_e( 'Product(s)', 'wcot-order-tags' ); ?><br>
										<select class="wcot-order-tags-select2-ajax-set-if-product-value" name="wcot_order_tags_set_if_product_value[]" multiple>
											<?php
											if ( !empty( $set_if_product_value ) ) {
												foreach ( $set_if_product_value as $set_if_product_value_id ) {
													?>
													<option value="<?php echo esc_html( $set_if_product_value_id ); ?>" selected><?php echo wp_kses_post( get_the_title( $set_if_product_value_id ) ) . ' ' . esc_html__( '-', 'wcot-order-tags' ) . ' ' . esc_html__( 'ID:', 'wcot-order-tags' ) . ' ' . esc_html( $set_if_product_value_id ); ?></option>
													<?php
												}
											}
											?>
										</select>
									</label>
								</div>
							</div>
							<div class="wcot-order-tags-checkbox-wrap">
								<label>
									<input type="checkbox" class="wcot-order-tags-checkbox-expanding" name="wcot_order_tags_set_if_product_category"<?php echo ( 'on' == $set_if_product_category ? ' checked' : '' ); ?>><?php esc_html_e( 'Product category', 'wcot-order-tags' ); ?>
								</label>
								<p class="description"><?php esc_html_e( 'Sets the tag if the order contains a product from a specific category(s).', 'wcot-order-tags' ); ?></p>
								<div class="wcot-order-tags-checkbox-expanded">
									<label>
										<?php esc_html_e( 'Product category(s)', 'wcot-order-tags' ); ?><br>
										<select class="wcot-order-tags-select2-ajax-set-if-product-category-value" name="wcot_order_tags_set_if_product_category_value[]" multiple>
											<?php
											if ( !empty( $set_if_product_category_value ) ) {
												foreach ( $set_if_product_category_value as $set_if_product_category_value_id ) {
													$set_if_product_category_value_term = get_term( $set_if_product_category_value_id );
													if ( !empty( $set_if_product_category_value_term ) ) {
														?>
														<option value="<?php echo esc_html( $set_if_product_category_value_id ); ?>" selected><?php echo wp_kses_post( $set_if_product_category_value_term->name ) . ' ' . esc_html__( '-', 'wcot-order-tags' ) . ' ' . esc_html__( 'ID:', 'wcot-order-tags' ) . ' ' . esc_html( $set_if_product_category_value_id ); ?></option>
														<?php
													}
												}
											}
											?>
										</select>
									</label>
								</div>
							</div>
							<div class="wcot-order-tags-checkbox-wrap">
								<label>
									<input type="checkbox" class="wcot-order-tags-checkbox-expanding" name="wcot_order_tags_set_if_product_meta_data"<?php echo ( 'on' == $set_if_product_meta_data ? ' checked' : '' ); ?>><?php esc_html_e( 'Product meta data', 'wcot-order-tags' ); ?>
								</label>
								<p class="description"><?php esc_html_e( 'Sets the tag if the order contains a product which has specific meta data(s).', 'wcot-order-tags' ); ?></p>
								<div class="wcot-order-tags-checkbox-expanded">
									<label>
										<?php esc_html_e( 'Product meta data(s)', 'wcot-order-tags' ); ?><br>
										<textarea name="wcot_order_tags_set_if_product_meta_data_value"><?php echo wp_kses_post( $set_if_product_meta_data_value ); ?></textarea>
									</label>
									<p class="wcot-order-tags-info"><?php echo wp_kses_post( $meta_data_field_info ); ?></p>
									<code>_application_id|contains|123
									_backorders|=|yes
									_weight|>|20
									</code><?php // This is intentionaly indented this way to display correctly ?>
									<p class="wcot-order-tags-info"><?php esc_html_e( 'If the product is a variation then the meta data is got from the variation, if the meta data is not found it will then attempt to get the meta data from the parent variable product instead.', 'wcot-order-tags' ); ?></p>
								</div>
							</div>
							<div class="wcot-order-tags-checkbox-wrap">
								<label>
									<input type="checkbox" class="wcot-order-tags-checkbox-expanding" name="wcot_order_tags_set_if_product_tag"<?php echo ( 'on' == $set_if_product_tag ? ' checked' : '' ); ?>><?php esc_html_e( 'Product tag', 'wcot-order-tags' ); ?>
								</label>
								<p class="description"><?php esc_html_e( 'Sets the tag if the order contains a product with a specific tag(s).', 'wcot-order-tags' ); ?></p>
								<div class="wcot-order-tags-checkbox-expanded">
									<label>
										<?php esc_html_e( 'Product tag(s)', 'wcot-order-tags' ); ?><br>
										<select class="wcot-order-tags-select2-ajax-set-if-product-tag-value" name="wcot_order_tags_set_if_product_tag_value[]" multiple>
											<?php
											if ( !empty( $set_if_product_tag_value ) ) {
												foreach ( $set_if_product_tag_value as $set_if_product_tag_value_id ) {
													$set_if_product_tag_value_term = get_term( $set_if_product_tag_value_id );
													if ( !empty( $set_if_product_tag_value_term ) ) {
														?>
														<option value="<?php echo esc_html( $set_if_product_tag_value_id ); ?>" selected><?php echo wp_kses_post( $set_if_product_tag_value_term->name ) . ' ' . esc_html__( '-', 'wcot-order-tags' ) . ' ' . esc_html__( 'ID:', 'wcot-order-tags' ) . ' ' . esc_html( $set_if_product_tag_value_id ); ?></option>
														<?php
													}
												}
											}
											?>
										</select>
									</label>
								</div>
							</div>
							<div class="wcot-order-tags-checkbox-wrap">
								<label>
									<input type="checkbox" class="wcot-order-tags-checkbox-expanding" name="wcot_order_tags_set_if_shipping_method"<?php echo ( 'on' == $set_if_shipping_method ? ' checked' : '' ); ?>><?php esc_html_e( 'Shipping method', 'wcot-order-tags' ); ?>
								</label>
								<p class="description"><?php esc_html_e( 'Sets the tag based on the shipping method(s) used.', 'wcot-order-tags' ); ?></p>
								<div class="wcot-order-tags-checkbox-expanded">
									<label>
										<?php esc_html_e( 'Shipping method(s)', 'wcot-order-tags' ); ?><br>
										<select class="wcot-order-tags-select2" name="wcot_order_tags_set_if_shipping_method_value[]" multiple>
											<?php foreach ( $shipping_methods as $shipping_method_id => $shipping_method ) { ?>
												<option value="<?php echo esc_html( $shipping_method_id ); ?>"<?php echo ( in_array( $shipping_method_id, $set_if_shipping_method_value ) ? ' selected' : '' ); ?>><?php echo esc_html( $shipping_method->method_title ) . ' ' . esc_html__( '-', 'wcot-order-tags' ) . ' ' . esc_html__( 'ID:', 'wcot-order-tags' ) . ' ' . esc_html( $shipping_method_id ); ?></option>
											<?php } ?>
										</select>
									</label>
								</div>
							</div>
							<div class="wcot-order-tags-checkbox-wrap">
								<label>
									<input type="checkbox" class="wcot-order-tags-checkbox-expanding" name="wcot_order_tags_set_if_shipping_method_instance"<?php echo ( 'on' == $set_if_shipping_method_instance ? ' checked' : '' ); ?>><?php esc_html_e( 'Shipping method instance', 'wcot-order-tags' ); ?>
								</label>
								<p class="description"><?php esc_html_e( 'Sets the tag based on the shipping method instance(s) used.', 'wcot-order-tags' ); ?></p>
								<div class="wcot-order-tags-checkbox-expanded">
									<label>
										<?php esc_html_e( 'Shipping method instance(s)', 'wcot-order-tags' ); ?><br>
										<select class="wcot-order-tags-select2" name="wcot_order_tags_set_if_shipping_method_instance_value[]" multiple>
											<?php
											foreach ( $shipping_method_instances as $shipping_method_instance ) {
												foreach ( $shipping_method_instance as $shipping_method_instance_object ) {
													?>
													<option value="<?php echo esc_html( $shipping_method_instance_object->instance_id ); ?>"<?php echo ( in_array( $shipping_method_instance_object->instance_id, $set_if_shipping_method_instance_value ) ? ' selected' : '' ); ?>><?php echo esc_html( $shipping_method_instance_object->title . ' ' . esc_html__( '-', 'wcot-order-tags' ) . ' ' . esc_html__( 'ID:', 'wcot-order-tags' ) . ' ' . esc_html( $shipping_method_instance_object->instance_id ) ); ?></option>
													<?php
												}
											}
											?>
										</select>
									</label>
								</div>
							</div>
							<div class="wcot-order-tags-checkbox-wrap">
								<label>
									<input type="checkbox" class="wcot-order-tags-checkbox-expanding" name="wcot_order_tags_set_if_shipping_zone"<?php echo ( 'on' == $set_if_shipping_zone ? ' checked' : '' ); ?>><?php esc_html_e( 'Shipping zone', 'wcot-order-tags' ); ?>
								</label>
								<p class="description"><?php esc_html_e( 'Sets the tag based on the shipping zone(s) used. Shipping zone(s) determined from the order\'s shipping method instance when order placed/updated.', 'wcot-order-tags' ); ?></p>
								<div class="wcot-order-tags-checkbox-expanded">
									<label>
										<?php esc_html_e( 'Shipping zone(s)', 'wcot-order-tags' ); ?><br>
										<select class="wcot-order-tags-select2" name="wcot_order_tags_set_if_shipping_zone_value[]" multiple>
											<?php foreach ( $shipping_zones as $shipping_zone ) { ?>
												<option value="<?php echo esc_html( $shipping_zone['id'] ); ?>"<?php echo ( in_array( $shipping_zone['id'], $set_if_shipping_zone_value ) ? ' selected' : '' ); ?>><?php echo esc_html( $shipping_zone['zone_name'] ) . ' ' . esc_html__( '-', 'wcot-order-tags' ) . ' ' . esc_html__( 'ID:', 'wcot-order-tags' ) . ' ' . esc_html( $shipping_zone['id'] ); ?></option>
											<?php } ?>		
										</select>
									</label>
								</div>
							</div>
						</td>
					</tr>
					<tr class="form-field">
						<th scope="row">
							<?php esc_html_e( 'Set If Mode', 'wcot-order-tags' ); ?>
						</th>
						<td>
							<label>
								<select name="wcot_order_tags_set_if_mode">
									<option value="all"<?php echo ( 'all' == $set_if_mode ? ' selected' : '' ); ?>><?php esc_html_e( 'All', 'wcot-order-tags' ); ?></option>
									<option value="any"<?php echo ( 'any' == $set_if_mode ? ' selected' : '' ); ?>><?php esc_html_e( 'Any', 'wcot-order-tags' ); ?></option>
								</select>
							</label>
							<p class="description">
								<?php esc_html_e( 'If set to all then if all of the "Set If" conditions enabled are met it sets the tag.', 'wcot-order-tags' ); ?><br>
								<?php esc_html_e( 'If set to any then if any of the "Set If" conditions enabled are met it sets the tag.', 'wcot-order-tags' ); ?>
							</p>
						</td>
				</tbody>
			</table>

			<?php

		}

		public function fields_select2_set_if_customer_value() {

			$customers = get_users(
				array(
					'role__in'	=> apply_filters( 'wcot_order_tags_customer_roles', array( 'customer' ) ),
					'order'		=> 'asc',
					'orderby'	=> 'display_name',
					'fields'	=> array(
						'ID',
						'display_name',
						'user_email',
					),
				)
			);

			$json = [];

			if ( !empty( $customers ) ) {

				foreach ( $customers as $customer ) {

					$json[] = array(
						'id'	=> $customer->ID,
						'text'	=> esc_html( $customer->display_name ) . ' ' . esc_html__( '-', 'wcot-order-tags' ) . ' ' . esc_html( $customer->user_email ) . ' ' . esc_html__( '-', 'wcot-order-tags' ) . ' ' . esc_html__( 'ID:', 'wcot-order-tags' ) . ' ' . esc_html( $customer->ID ),
					);

				}

			}

			echo wp_json_encode( $json );

			exit;

		}

		public function fields_select2_set_if_product_value() {

			$products = wc_get_products(
				array(
					'limit'		=> -1,
					'order'		=> 'asc',
					'orderby'	=> 'name',
					'return'	=> 'ids',
				)
			);

			$json = [];

			if ( !empty( $products ) ) {

				foreach ( $products as $product_id ) {

					$json[] = array(
						'id'	=> $product_id,
						'text'	=> wp_kses_post( get_the_title( $product_id ) ) . ' ' . esc_html__( '-', 'wcot-order-tags' ) . ' ' . esc_html__( 'ID:', 'wcot-order-tags' ) . ' ' . esc_html( $product_id ),
					);

				}

			}

			echo wp_json_encode( $json );

			exit;

		}

		public function fields_select2_set_if_product_category_value() {

			$product_categories = get_terms(
				'product_cat',
				array(
					'orderby'    => 'title',
					'order'      => 'asc',
					'hide_empty' => false,
				)
			);

			$json = [];

			if ( !empty( $product_categories ) ) {

				foreach ( $product_categories as $product_category ) {

					$json[] = array(
						'id'	=> $product_category->term_id,
						'text'	=> wp_kses_post( $product_category->name ) . ' ' . esc_html__( '-', 'wcot-order-tags' ) . ' ' . esc_html__( 'ID:', 'wcot-order-tags' ) . ' ' . esc_html( $product_category->term_id ),
					);

				}

			}

			echo wp_json_encode( $json );

			exit;

		}

		public function fields_select2_set_if_product_tag_value() {

			$product_tags = get_terms(
				'product_tag',
				array(
					'orderby'    => 'title',
					'order'      => 'asc',
					'hide_empty' => false,
				)
			);

			$json = [];

			if ( !empty( $product_tags ) ) {

				foreach ( $product_tags as $product_tag ) {

					$json[] = array(
						'id'	=> $product_tag->term_id,
						'text'	=> wp_kses_post( $product_tag->name ) . ' ' . esc_html__( '-', 'wcot-order-tags' ) . ' ' . esc_html__( 'ID:', 'wcot-order-tags' ) . ' ' . esc_html( $product_tag->term_id ),
					);

				}

			}

			echo wp_json_encode( $json );

			exit;

		}

		public function save( $term_id ) {

			global $wpdb;

			if ( !empty( $_POST ) ) {

				if ( isset( $_POST['wcot_order_tags_save_nonce'] ) ) { // This also ensures we are only dealing with edits to the order tags and not effecting other taxonomies

					if ( wp_verify_nonce( sanitize_key( $_POST['wcot_order_tags_save_nonce'] ), 'wcot_order_tags_save' ) ) {

						// Delete all order tag meta - this ensures this if an option was previously populated and no longer is and isn't submitted such as checkboxes, the removal occurs. Note the deletion used to be done via a $wpdb delete from query but this was replaced with the below due to issues on some websites with data being removed on save (possibly due to the wpdb query taking longer to run than the foreach $_POST below)

						$order_tag_metas = get_term_meta( $term_id );

						if ( !empty( $order_tag_metas ) ) {

							foreach ( $order_tag_metas as $order_tag_meta_key => $order_tag_meta_values ) {

								if ( strpos( $order_tag_meta_key, 'wcot_order_tags_' ) === 0 ) {

									delete_term_meta( $term_id, $order_tag_meta_key );

								}

							}

						}

						// Loop all $_POST

						foreach ( $_POST as $post_key => $post_value ) {

							// If order tag field

							if ( strpos( $post_key, 'wcot_order_tags_' ) === 0 ) {

								// If not the nonce field as don't want this saved as meta

								if ( 'wcot_order_tags_save_nonce' !== $post_key ) {

									// Add the order tag term meta

									update_term_meta( $term_id, $post_key, $post_value );

								}

							}

						}

					}

				}

			}

		}

		public function columns( $columns ) {

			unset( $columns['slug'] );
			unset( $columns['posts'] );
			return $columns;

		}

	}

}
